<?php
// wbb imports
require_once(WBB_DIR.'lib/data/boxes/PortalHelpers.class.php');

// wcf imports
require_once(WCF_DIR.'lib/data/DatabaseObject.class.php');

/**
 * Represents a portal box database row
 *
 * @package	com.woltlab.community.portal
 * @author	WoltLab Community
 * @copyright	2009-2010 WoltLab Community
 * @license	GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	data.boxes
 * @category	Portal
 */
class PortalBox extends DatabaseObject {
	/**
	 * Contains all available portal boxes
	 * Load the portal boxes with PortalBox::loadBoxes
	 *
	 * @see	PortalBox::loadBoxes()
	 * @var array<PortalBox>
	 */
	protected static $boxes = null;

	/**
	 * Stores whether this box has been initalized yet
	 *
	 * @var boolean
	 */
	protected $initalized = false;

	/**
	 * True if there is no data to be shown
	 *
	 * @var boolean
	 */
	protected $empty = false;

	/**
	 * False if box is disabled by user, this field is *NOT* stored in database
	 *
	 * @var	boolean
	 */
	protected $enabled = true;

	/**
	 * Contains the cached box data
	 * Class variable as workaround for the bugged PHP 5.2.0 version
	 *
	 * @var array
	 */
	public $cacheData = array();

	/**
	 * Contains stylesheet for this box
	 *
	 * @var	string
	 */
	public $stylesheet = '';

	/**
	 * Includes an IE stylesheet
	 *
	 * @var	boolean
	 */
	public $includeIEStylesheet = false;

	/**
	 * Creates a new PortalBox object.
	 *
	 * If id is set, the function reads the portal box data from database.
	 * Otherwise it uses the given resultset.
	 *
	 * @param 	integer		$boardID		id of a portal box
	 * @param 	array		$row			resultset with portal box data from database
	 * @param 	PortalBox 		$cacheObject	object with portal box data from database
	 */
	public function __construct($boxID, $row = null, $cacheObject = null) {
		if ($boxID !== null) $cacheObject = self::getBox($boxID);
		if ($row !== null) parent::__construct($row);
		if ($cacheObject !== null) parent::__construct($cacheObject->data);
	}

	/**
	 * Reads all necessary data from cache
	 *
	 * @param	boolean	$force Should a reinitialization be forced
	 * @return 	void
	 */
	public function initialize($force = false) {
		// register stylesheet
		PortalHelpers::addBoxStylesheet($this->stylesheet, $this->includeIEStylesheet);

		if (($this->useCache && !$this->initalized) || ($this->useCache && $force)) {

			if (!file_exists($this->cacheTypeClassFile)) {
				throw new SystemException("Unable to find class file '".$this->cacheTypeClassFile."'", 11000);
			}
			require_once($this->cacheTypeClassFile);

			if (!class_exists($this->cacheTypeClassName)) {
				throw new SystemException("Unable to find class '".$this->cacheTypeClassName."'", 11001);
			}
			$className 	= $this->cacheTypeClassName;
			$boxObj		= new $className;

			// get cached box data
			$this->cacheData = $boxObj->execute(
				$this->boxID,
				$this->cacheBuilderClass,
				'type'.$this->boxType,
				$this->minLifeTime,
				$this->maxLifeTime
			);
			$this->initalized = true;
		}
	}

	/**
	 * Returns the name of the user option/session status variable
	 *
	 * @return string
	 */
	public function getStatusVariable() {
		return $this->boxName.$this->boxID;
	}

	/**
	 * Returns whether this box is opened or closed
	 *
	 * @param $additionalData
	 * @return integer
	 */
	public function getStatus($additionalData = array()) {
		// get box status
		$status = 1;
		$statusName = $this->getStatusVariable();
		if (WCF::getUser()->userID) {
			$status = intval(WCF::getUser()->getUserOption($statusName));
		}
		else {
			if (WCF::getSession()->getVar($statusName) != false) {
				$status = WCF::getSession()->getVar($statusName);
			}
		}

		return $status;
	}

	/**
	 * Returns whether this box is empty or not
	 *
	 * @return boolean
	 */
	public function isEmpty() {
		return $this->empty;
	}

	/**
	 * Returns whether this box should be displayed or not
	 *
	 * @return boolean
	 */
	public function isVisible() {
		if ($this->isEmpty() && PORTAL_SHOW_EMPTY_BOXES == false) {
			return false;
		}
		else return true;
	}

	/**
	 * Clears the data cache of this box
	 *
	 * @return void
	 */
	public function clearDataCache() {
		WCF::getCache()->clear(WBB_DIR . 'cache/', 'cache.box-'.$this->boxID.'*');
	}

	/**
	 * Disables a box
	 */
	public function disableBox() {
		$this->enabled = false;
	}

	/**
	 * Returns wether box is enabled or disabled by user
	 *
	 * @return	boolean
	 */
	public function isEnabled() {
		return $this->enabled;
	}

	/**
	 * Fetches the portal box with the given box id from cache.
	 *
	 * @param 	integer		$boxID	id of the requested box
	 * @return	PortalBox
	 */
	public static function getBox($boxID) {
		if (self::$boxes === null) {
			self::$boxes = self::loadBoxes();
		}

		foreach (self::$boxes as $box) {
			if ($box->boxID == $boxID) return $box;
		}

		throw new IllegalLinkException();
	}

	/**
	 * Fetches the portal box with the given box name from cache.
	 *
	 * @param 	string		$boxName	name of the requested box
	 * @return	PortalBox
	 */
	public static function getBoxByName($boxName) {
		if (self::$boxes === null) {
			self::$boxes = self::loadBoxes();
		}

		foreach (self::$boxes as $box) {
			if ($box->boxName == $boxName) return $box;
		}

		throw new IllegalLinkException();
	}

	/**
	 * Gets the boxes from cache
	 *
	 * @return array<PortalBox>
	 */
	public static function loadBoxes() {
		if (self::$boxes == null) {
			// load classes before unserializing the portal box classes
			WCF::getCache()->addResource('portalBoxNames', WBB_DIR.'cache/cache.portalBoxNames.php', WBB_DIR.'lib/system/cache/CacheBuilderPortalBoxNames.class.php');
			$boxes = WCF::getCache()->get('portalBoxNames');

			foreach ($boxes as $box) {
				if (!file_exists($box['boxClassFile'])) {
					throw new SystemException("Unable to find class file '".$box['boxClassFile']."'", 11000);
				}
				require_once($box['boxClassFile']);

				if (!class_exists($box['className'])) {
					throw new SystemException("Unable to find class '".$box['className']."'", 11001);
				}
			}

			WCF::getCache()->addResource('portalBoxes', WBB_DIR.'cache/cache.portalBoxes.php', WBB_DIR.'lib/system/cache/CacheBuilderPortalBoxes.class.php');
			self::$boxes = WCF::getCache()->get('portalBoxes');
		}

		return self::$boxes;

	}

	/**
	 * Resets the portal box cache after changes.
	 *
	 * @param	boolean	$clearDataCache Clears the data cache of all boxes
	 */
	public static function resetCache($clearDataCache = false) {
		if ($clearDataCache) {
			if (self::$boxes === null) {
				self::$boxes = self::loadBoxes();
			}
			foreach (self::$boxes as $box) {
				WCF::getCache()->clear(WBB_DIR . 'cache/', 'cache.box-'.$box->boxID.'*');
			}
		}
		try {
			// reset cache
			WCF::getCache()->clearResource('portalBoxes');
			WCF::getCache()->clearResource('portalBoxNames');
		}
		catch (SystemException $ex) {
			WCF::getCache()->clear(WBB_DIR.'cache/', 'cache.portal*');
		}

		self::$boxes = null;
	}
}
?>