<?php
/**
 * Provides common data for portal boxes
 *
 * @package	com.woltlab.community.portal.helper
 * @author	Alexander Ebert
 * @copyright	2009-2011 WoltLab Community
 * @license	GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	data.boxes
 * @category	Portal
 */
class PortalHelpers {
	/**
	 * Stores all box-related stylesheets
	 *
	 * @var	array<array>
	 */
	protected static $boxStylesheets = array(
		'default' => array(),
		'ie' => array()
	);

	protected static $caches = array(
		'news'
	);

	/**
	 * Stores compilant charset names
	 *
	 * @var	array
	 */
	protected static $charsets = array(
		'utf8' => 'UTF-8',
		'latin1' => 'ISO-8859-1',
		'latin2' => 'ISO-8859-2',
		'greek' => 'ISO-8859-7',
		'hebrew' => 'ISO-8859-8',
		'latin5' => 'ISO-8859-9',
		'sjis' => 'SJIS',
		'ujis' => 'EUC-JP',
		'big5' => 'BIG-5',
		'gb2312' => 'EUC-CN',
		'gbk' => 'CP936',
		'koi8r' => 'KOI8-R',
		'cp1251' => 'Windows-1251',
		'euckr' => 'EUC-KR'
	);

	protected static $currentEncoding = '';

	/**
	 * Stores user online list indexed by userID
	 *
	 * @var	array
	 */
	protected static $usersOnlineList = array();

	/**
	 * Initializes system-wide portal environment variables
	 */
	public static function init() {
		// handle different WBB versions
		if (version_compare(PACKAGE_VERSION, '3.1.0 Beta 1', '<')) {
			// WBBLite 2.1.x
			if(!defined('PORTAL_COMPATIBILITY_MODE')) define('PORTAL_COMPATIBILITY_MODE', 1);
		}
		else {
			// WBB 3.1.x
			if(!defined('PORTAL_COMPATIBILITY_MODE')) define('PORTAL_COMPATIBILITY_MODE', 0);
		}
	}

	/**
	 * Returns a compact list of all online users
	 *
	 * @param	boolean	$returnData
	 * @return	array<array>
	 */
	public static function getUsersOnlineList($returnData = true) {
		// not supported in WBBLite 2.1.x
		if (PORTAL_COMPATIBILITY_MODE == 1) return null;

		require_once(WBB_DIR.'lib/data/user/usersOnline/PortalUsersOnlineList.class.php');

		if (empty(self::$usersOnlineList)) {
			$cuol = new PortalUsersOnlineList();
			self::$usersOnlineList = $cuol->getUsersOnline();
		}

		if ($returnData) return self::$usersOnlineList;
	}

	/**
	 * Verifies if a user is online and return formatted username if requested. This
	 * method will always return bool(false) if user is offline
	 *
	 * @param	integer	$userID
	 * @param	boolean	$returnFormattedUsername
	 * @return	mixed
	 */
	public static function getUserOnlineStatus($userID, $returnFormattedUsername = false) {
		// not supported in WBBLite 2.1.x
		if (PORTAL_COMPATIBILITY_MODE == 1) return null;

		if (empty(self::$usersOnlineList)) self::getUsersOnlineList(false);

		// check if user is listed as online
		if (array_key_exists(intval($userID), self::$usersOnlineList)) {
			if ($returnFormattedUsername) {
				return self::$usersOnlineList[intval($userID)];
			}

			return true;
		}

		return false;
	}

	/**
	 * Add a box-related stylesheet
	 *
	 * @param	string	$name	Stylesheet name without file extension, will store lower-case name
	 * @param	boolean	$isIE	Stylesheet is specially crafted for Internet Explorer < 8
	 */
	public static function addBoxStylesheet($name, $includeIEStylesheet = false) {
		$name = StringUtil::toLowerCase($name);

		if (empty($name)) return;

		if (!in_array($name, self::$boxStylesheets['default'])) {
			self::$boxStylesheets['default'][] = $name;

			if ($includeIEStylesheet && !in_array($name, self::$boxStylesheets['ie'])) {
				self::$boxStylesheets['ie'][] = $name;
			}
		}
	}

	/**
	 * Returns a list of all box stylesheets
	 *
	 * @return	string	Stylesheets
	 */
	public static function getBoxStylesheets() {
		$stylesheets = '';

		// include default css
		$stylesheets .= "\n".'<link rel="stylesheet" type="text/css" media="screen" href="index.php?page=PortalCSS&amp;names='.implode(',', self::$boxStylesheets['default']).'" />';

		// if applicable insert IE stylesheets after "normal" stylesheets
		if (!empty(self::$boxStylesheets['ie'])) {
			$stylesheets .= "\n".'<!--[if lt IE 8]>';
			$stylesheets .= "\n".'<link rel="stylesheet" type="text/css" media="screen" href="index.php?page=PortalCSS&amp;ie&amp;names='.implode(',', self::$boxStylesheets['ie']).'" />';
			$stylesheets .= "\n".'<![endif]-->';
		}

		return $stylesheets;
	}

	/**
	 * Converts non-utf-8 encoded strings
	 *
	 * @param	string	$string	Input string
	 * @return	string		UTF-8 encoded string
	 */
	public static function convertEncoding($string) {
		$encoding = self::getDatabaseEncoding();

		if (array_key_exists($encoding, self::$charsets)) {
			$encoding = self::$charsets[$encoding];
		} else {
			$encoding = 'UTF-8';
		}

		if ($encoding == 'UTF-8') {
			return $string;
		}
		else {
			return StringUtil::convertEncoding($encoding, 'UTF-8', $string);
		}
	}

	/**
	 * Reads current database encoding
	 *
	 * @return	string	Encoding
	 */
	public static function getDatabaseEncoding() {
		if (empty(self::$currentEncoding)) {
			self::$currentEncoding = WCF::getDB()->getCharset();
		}

		return self::$currentEncoding;
	}

	/**
	 * Deletes a cache resource
	 *
	 * @param	string	$cacheName	Cache Resource
	 */
	public static function deleteCache($cacheName) {
		$cacheName = StringUtil::toLowerCase($cacheName);
		if (!in_array($cacheName, self::$caches)) return;

		WCF::getCache()->addResource(
			'portal'.ucfirst($cacheName),
			WBB_DIR.'lib/cache.portal-'.$cacheName.'.php',
			WBB_DIR.'lib/system/cache/CacheBuilderPortalNews.class.php'
		);

		WCF::getCache()->clearResource('portal'.ucfirst($cacheName));
		WCF::getCache()->clear(WBB_DIR.'cache/', 'cache.portal-'.$cacheName.'.php');
	}
}
?>