<?php
// wbb imports
require_once(WBB_DIR.'lib/data/boxes/PortalBox.class.php');

// wcf imports
require_once(WCF_DIR.'lib/system/event/EventHandler.class.php');

/**
 * Loads and stores portal boxes
 *
 * @package	com.woltlab.community.portal
 * @author	WoltLab Community
 * @copyright	2009-2010 WoltLab Community
 * @license	GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	system.portalbox
 * @category	Portal
 */
class PortalBoxFactory {
	/**
	 * Contains the boxes
	 *
	 * @var array<mixed>
	 */
	protected $data	= array(
		'type1' => array(),
		'type2' => array(),
		'type3' => array()
	);

	/**
	 * Fetch all available boxes and render them
	 *
	 * @param 	boolean	$skipInactive 		Defaults to true. Doesn't load inactive boxes
	 * @param	boolean	$displayInForum		Defaults to false. Loads only boxes which should be shown in the forum
	 * @param	boolean	$initializeBoxes	Defaults to false. If true, all boxes will be initialized and read their data
	 * @param	boolean	$applyUserSort		Defaults to false. If true, user sort will be applied if applicable
	 * @param	boolean	$skipUserDeactivated	Defaults to false. If true, user deactivated boxes will not be loaded
	 * @param	boolean	$acpview		Defaults to false. If true, permissions for portalboxes will be ignored
	 * @param	boolean	$showAllOnEditForm	Defaults to false. If true, all boxes available to the user will be loaded in the edit form (empty boxes, too!)
	 */
	public function __construct($skipInactive = true, $displayInForum = false, $initializeBoxes = false, $applyUserSort = false, $skipUserDeactivated = false, $acpview = false, $showAllOnEditForm = false) {
		$this->selectBoxes($skipInactive, $displayInForum, $acpview);

		if ($initializeBoxes) $this->initializeBoxes($showAllOnEditForm);
		if ($applyUserSort) $this->applyUserSort($skipUserDeactivated);
	}

	/**
	 * Fetch available boxes
	 *
	 * @param	boolean		$skipInactive	Ignores deactivated boxes
	 * @param	boolean		$displayInForum	Exclude center boxes
	 * @see		__construct
	 */
	protected function selectBoxes($skipInactive, $displayInForum, $acpview) {
		$boxes = PortalBox::loadBoxes();

		foreach ($boxes as $box) {
			// ignore inactive boxes
			if ($skipInactive && $box->boxActivation != 1) continue;

			// ignore center boxes if in board. also only show boxes that are enabled for display in board
			if ($displayInForum && ($box->boxType == 1 || !$box->showInForum)) continue;
			
			//ignore permissions if loaded in ACP
			if (!$acpview) {
				$permissions = $box->permissions;
				// check permissions
				if (!empty($permissions)) {
					$permissions = explode(',', $permissions);
					foreach ($permissions as $permission) {
						if (!WCF::getUser()->getPermission($permission)) continue 2;
					}
				}
			}
			

			$options = $box->options;
			// check module system
			if (!empty($options)) {
				$options = explode(',', StringUtil::toUpperCase($options));
				foreach ($options as $option) {
					if (!defined($option) || !constant($option)) continue 2;
				}
			}
			$this->data['type'.$box->boxType][$box->boxID] = $box;
		}
	}

	/**
	 * Intialize all fetched boxes
	 *
	 * @see		__construct
	 */
	protected function initializeBoxes($showAllOnEditForm) {
		if (!empty($this->data)) {
			// run through all boxtypes
			foreach ($this->data as $boxType => $boxes) {
				// handle each assigned boxID
				foreach ($boxes as $boxID => $box) {
					// initialize boxes
					$box->initialize();
					EventHandler::fireAction($box, 'initalized');

					// read box data
					$box->readData();
					EventHandler::fireAction($box, 'dataRead');

					// remove boxes which are not visible to the user
					if ($box->isVisible() == false && !$showAllOnEditForm) {
						unset($this->data[$boxType][$boxID]);
					}
				}
			}
		}
	}

	/**
	 * Returns all boxes which are displayed in center
	 *
	 * @return	array
	 */
	public function getType1Data() {
		if (isset($this->data['type1'])) {
			return $this->data['type1'];
		}

		return array();
	}

	/**
	 * Returns all boxes which are display on the left hand
	 *
	 * @return	array
	 */
	public function getType2Data() {
		if (isset($this->data['type2'])) {
			return $this->data['type2'];
		}

		return array();
	}

	/**
	 * Returns all boxes which are display on the right hand
	 *
	 * @return	array
	 */
	public function getType3Data() {
		if (isset($this->data['type3'])) {
			return $this->data['type3'];
		}

		return array();
	}

	/**
	 * Returns which columns are active, thus containing at least one box
	 *
	 * @return	array
	 */
	public function getColumnStatus() {
		$left = (empty($this->data['type2'])) ? false : true;
		$right = (empty($this->data['type3'])) ? false : true;

		return array('left' => $left, 'right' => $right);
	}

	/**
	 * Sort boxes against user settings if applicable
	 *
	 * @see	__construct()
	 */
	protected function applyUserSort($skip) {
		// ignore guests
		if (!WCF::getUser()->userID) return;

		// ignore users with no own box sort
		if (!WCF::getUser()->customPortalBoxSort) return;

		// fetch sort data
		$sql = "SELECT	*
			FROM	wbb".WBB_N."_portalboxes_sort_to_user
			WHERE	userID = ".WCF::getUser()->userID;
		$sortData = WCF::getDB()->getFirstRow($sql);

		// rebuild array data
		$sortData = array(
			1 => unserialize($sortData['centerboxes']),
			2 => unserialize($sortData['leftboxes']),
			3 => unserialize($sortData['rightboxes'])
		);

		// copy original data and clear data field
		$originalData = array(
			'type1' => array(),
			'type2' => array(),
			'type3' => array()
		);
		$originalData = array_merge($originalData, $this->data);
		$this->data = array(
			'type1' => array(),
			'type2' => array(),
			'type3' => array()
		);

		// handle centerboxes
		$this->sortCenterBoxes($originalData['type1'], $sortData[1], $skip);
		unset($originalData['type1']);
		unset($sortData[1]);

		// handle left and right column
		$this->sortRemainingBoxes($originalData, $sortData, $skip);
	}

	/**
	 * Sorts center boxes, they are special in a way these boxes cannot be move
	 * to left or right column
	 *
	 * @param	array	$originalData
	 * @param	array	$sortData
	 */
	protected function sortCenterBoxes($originalData, $sortData, $skip) {
		// Workaround, when unserialize returns boolean(false) in PortalBoxFactory::applyUserSort()
		if (!is_array($sortData)) $sortData = array();
		
		foreach ($sortData as $boxID => $enabled) {
			// skip unknown boxID
			if (!isset($originalData[$boxID])) continue;

			// disable box if applicable
			if (!$enabled) $originalData[$boxID]->disableBox();

			// add box if active or inside PortalEditForm
			if ($enabled || !$skip) {
				$this->data['type1'][$boxID] = $originalData[$boxID];
			}

			// remove box from data
			unset ($originalData[$boxID]);
		}

		// append remaining boxes
		if (!empty($originalData)) {
			$this->data['type1'] = array_merge($this->data['type1'], $originalData);
		}
	}

	/**
	 * Sorts left and right column
	 *
	 * @param	array	$originalData
	 * @param	array	$sortData
	 */
	protected function sortRemainingBoxes($originalData, $sortData, $skip) {
		// Workaround, when unserialize returns boolean(false) in PortalBoxFactory::applyUserSort()
		if (!is_array($sortData)) $sortData = array();
		
		foreach ($sortData as $column => $boxes) {
			foreach ($boxes as $boxID => $enabled) {
				$originalColumn = null;

				// try to find box
				if (isset($originalData['type2'][$boxID])) {
					$originalColumn = 'type2';
				}
				else if (isset($originalData['type3'][$boxID])) {
					$originalColumn = 'type3';
				}

				// ignore unknown boxID
				if ($originalColumn === null) continue;

				// disable box if applicable
				if (!$enabled) $originalData[$originalColumn][$boxID]->disableBox();

				// add box if active or inside PortalEditForm
				if ($enabled || !$skip) {
					$this->data['type'.$column][$boxID] = $originalData[$originalColumn][$boxID];
				}

				// remove box from data
				unset ($originalData[$originalColumn][$boxID]);
			}
		}

		// append remaining boxes
		foreach ($originalData as $column => $boxes) {
			if (empty($boxes)) continue;

			$this->data[$column] = array_merge($this->data[$column], $boxes);
		}
	}
}
?>